<?php

namespace Drupal\exception_mailer\Form;

use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\user\RoleInterface;
use Drupal\user\RoleStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ExceptionMailerConfigForm.
 *
 * Provides a form to configure Error & Exception mailer settings.
 */
class ExceptionMailerConfigForm extends ConfigFormBase {

  /**
   * The role storage.
   *
   * @var \Drupal\user\RoleStorageInterface
   */
  protected $roleStorage;

  /**
   * The email validator.
   *
   * @var \Drupal\Component\Utility\EmailValidatorInterface
   */
  protected $emailValidator;

  /**
   * Constructs a \Drupal\exception_mailer\ExceptionMailerConfigForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\user\RoleStorageInterface $role_storage
   *   The role storage.
   * @param \Drupal\Component\Utility\EmailValidatorInterface $email_validator
   *   The email validator.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TypedConfigManagerInterface $typedConfigManager, RoleStorageInterface $role_storage, EmailValidatorInterface $email_validator) {
    parent::__construct($config_factory, $typedConfigManager);
    $this->roleStorage = $role_storage;
    $this->emailValidator = $email_validator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('entity_type.manager')->getStorage('user_role'),
      $container->get('email.validator')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      'exception_mailer.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'exception_mailer_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('exception_mailer.settings');
    $form['level_type'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Check the level type'),
      '#options' => RfcLogLevel::getLevels(),
      '#default_value' => $config->get('level_type'),
    ];

    $roles = $this->roleStorage->loadMultiple();
    unset($roles[RoleInterface::AUTHENTICATED_ID], $roles[RoleInterface::ANONYMOUS_ID]);
    $roles = array_map(fn(RoleInterface $role) => Html::escape($role->label()), $roles);

    $form['roles'] = [
      '#type' => 'select',
      '#title' => $this->t('Roles'),
      '#description' => $this->t('Email will be sent for users assigned to select role(s).'),
      '#options' => $roles,
      '#size' => min(count($roles), 8),
      '#multiple' => TRUE,
      '#default_value' => $config->get('roles'),
    ];

    $form['emails'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Email recipient addresses'),
      '#description' => $this->t('Email will be sent for entered emails. For more than one email separate it by comma.'),
      '#default_value' => $config->get('emails'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {

    // If both the Roles and Emails fields are empty than show the error.
    if (empty($form_state->getValue('roles')) && empty($form_state->getValue('emails'))) {
      $form_state->setErrorByName('error', $this->t('Please configure atleast one field Roles or Email address.'));
    }
    //Validate the email address field.
    if (!empty($form_state->getValue('emails'))) {
      $valid = [];
      $invalid = [];
      foreach (explode(",", trim($form_state->getValue('emails'))) as $email) {
        $email = trim($email);
        if (!empty($email)) {
          if ($this->emailValidator->isValid($email)) {
            $valid[] = $email;
          }
          else {
            $invalid[] = $email;
          }
        }
      }
      if (count($invalid) == 1) {
        $form_state->setErrorByName('emails', $this->t('%email is not a valid email address.', ['%email' => reset($invalid)]));
      }
      elseif (count($invalid) > 1) {
        $form_state->setErrorByName('emails', $this->t('%emails are not valid email addresses.', ['%emails' => implode(', ', $invalid)]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    parent::submitForm($form, $form_state);

    $this->config('exception_mailer.settings')
      ->set('level_type', array_filter($form_state->getValue('level_type'), function ($v) {
        return $v !== 0;
      }))
      ->set('roles', $form_state->getValue('roles'))
      ->set('emails', $form_state->getValue('emails'))
      ->save();
  }

}
